import alabaster
import vetoes

project = 'vetoes'
copyright = 'AWeber Communications, Inc.'
version = vetoes.version
release = '.'.join(str(v) for v in vetoes.version_info[:2])

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.intersphinx']
master_doc = 'index'
source_suffix = '.rst'

pygments_style = 'sphinx'
html_sidebars = {'**': []}
html_static_path = ['.']
intersphinx_mapping = {
    'python': ('http://docs.python.org/3/', None),
    'rejected': ('http://rejected.readthedocs.io/en/latest/', None),
    'tornado': ('http://tornadoweb.org/en/stable/', None),
}



import json
import socket

from rejected import consumer, testing
from tornado import concurrent, gen
import mock

from vetoes import config, service


class Consumer(service.HTTPServiceMixin,
               consumer.Consumer):

    def __init__(self, *args, **kwargs):
        kwargs['service_map'] = {'fetch-stats': 'httpbin'}
        super(Consumer, self).__init__(*args, **kwargs)
        self.method = 'GET'
        self.request_body = None
        self.request_json = None

    @gen.coroutine
    def process(self):
        yield self.call_http_service('fetch-stats', self.method, 'stats',
                                     **{'body': self.request_body,
                                        'json': self.request_json})

    def get_service_url(self, service, *path, **kwargs):
        return 'http://httpbin.org/status/200'


class ConfigurableConsumer(config.TimeoutConfigurationMixin, Consumer):
    pass


class HTTPServiceMixinTests(testing.AsyncTestCase):

    def setUp(self):
        super(HTTPServiceMixinTests, self).setUp()
        self.consumer.http = mock.Mock()
        self.http_response = mock.Mock(code=200, request_time=0)
        self.consumer.http.fetch.return_value = concurrent.Future()
        self.consumer.http.fetch.return_value.set_result(self.http_response)

    def get_consumer(self):
        return Consumer

    @testing.gen_test
    def test_that_sentry_context_is_managed(self):
        with mock.patch.multiple(self.consumer,
                                 set_sentry_context=mock.DEFAULT,
                                 unset_sentry_context=mock.DEFAULT) as context:
            yield self.process_message()
            context['set_sentry_context'].assert_called_once_with(
                'service_invoked', 'httpbin')
            context['unset_sentry_context'].assert_called_once_with(
                'service_invoked')

    @testing.gen_test
    def test_that_metrics_are_emitted(self):
        measurement = yield self.process_message()
        self.assertIn('http.fetch-stats.200', measurement.values)
        self.assertEqual(measurement.values['http.fetch-stats.200'],
                         self.http_response.request_time)

    @testing.gen_test
    def test_that_timeout_result_in_processing_exceptions(self):
        self.http_response.code = 599
        with self.assertRaises(consumer.ProcessingException):
            measurement = yield self.process_message()
            self.assertEqual(measurement.values['http.fetch-stats.599'],
                             self.http_response.request_time)

    @testing.gen_test
    def test_that_rate_limiting_result_in_processing_exceptions(self):
        self.http_response.code = 429
        with self.assertRaises(consumer.ProcessingException):
            measurement = yield self.process_message()
            self.assertEqual(measurement.values['http.fetch-stats.429'],
                             self.http_response.request_time)

    @testing.gen_test
    def test_that_call_http_service_accepts_body(self):
        self.consumer.method = 'POST'
        self.consumer.request_body = mock.sentinel.body
        yield self.process_message()
        self.consumer.http.fetch.assert_called_once_with(
            self.consumer.get_service_url('fetch-stats'),
            headers={'Correlation-Id': self.correlation_id},
            method='POST', body=mock.sentinel.body, raise_error=False)

    @testing.gen_test
    def test_that_call_http_service_jsonifies(self):
        self.consumer.method = 'POST'
        self.consumer.request_json = {'one': 1}
        yield self.process_message()
        self.consumer.http.fetch.assert_called_once_with(
            self.consumer.get_service_url('fetch-stats'),
            method='POST', body=json.dumps({'one': 1}).encode('utf-8'),
            headers={'Content-Type': 'application/json',
                     'Correlation-Id': self.correlation_id},
            raise_error=False)

    @testing.gen_test
    def test_that_socket_errors_result_in_processing_exception(self):
        future = concurrent.Future()
        future.set_exception(socket.error(42, 'message'))
        self.consumer.http.fetch.return_value = future

        with self.assertRaises(consumer.ProcessingException):
            yield self.process_message()
        self.assertGreater(
            self.consumer._measurement.values['http.fetch-stats.timeout'],
            self.http_response.request_time)
        self.assertEqual(
            self.consumer._measurement.counters['errors.socket.42'], 1)

    @testing.gen_test
    def test_that_raise_error_can_be_overridden(self):
        self.http_response.code = 500
        self.http_response.rethrow.side_effect = RuntimeError

        response = yield self.consumer.call_http_service(
            'fetch-stats', 'GET', raise_error=False)

        self.consumer.http.fetch.assert_called_once_with(
            self.consumer.get_service_url('fetch-stats'),
            method='GET', raise_error=False)
        self.assertIs(response, self.http_response)

    @testing.gen_test
    def test_that_url_kwarg_skips_service_lookup(self):
        with mock.patch.multiple(self.consumer,
                                 set_sentry_context=mock.DEFAULT,
                                 unset_sentry_context=mock.DEFAULT) as context:
            response = yield self.consumer.call_http_service(
                'frobinicate', 'GET', url='https://google.com')
            self.consumer.http.fetch.assert_called_once_with(
                'https://google.com', method='GET', raise_error=False)
            self.assertIs(response, self.http_response)
        context['set_sentry_context'].assert_called_once_with(
            'service_invoked', 'frobinicate')
        context['unset_sentry_context'].assert_called_once_with(
            'service_invoked')


class TimeoutConfiguredTests(testing.AsyncTestCase):

    def setUp(self):
        super(TimeoutConfiguredTests, self).setUp()
        self.consumer.http = mock.Mock()
        self.http_response = mock.Mock(code=200, request_time=0)
        self.consumer.http.fetch.return_value = concurrent.Future()
        self.consumer.http.fetch.return_value.set_result(self.http_response)

    def get_consumer(self):
        return ConfigurableConsumer

    @testing.gen_test
    def test_that_timeout_is_passed_through(self):
        yield self.process_message()
        self.consumer.http.fetch.assert_called_once_with(
            mock.ANY, headers=mock.ANY, method=mock.ANY, body=mock.ANY,
            raise_error=False,
            request_timeout=self.consumer.get_timeout('default'))

    @testing.gen_test
    def test_that_timeout_can_be_configured_by_function(self):
        self.consumer.settings.setdefault('timeouts', {})
        self.consumer.settings['timeouts']['fetch-stats'] = 1234.5
        yield self.process_message()
        self.consumer.http.fetch.assert_called_once_with(
            mock.ANY, headers=mock.ANY, method=mock.ANY, body=mock.ANY,
            raise_error=False, request_timeout=1234.5)

    @testing.gen_test
    def test_that_timeout_can_be_configured_by_service(self):
        self.consumer.settings.setdefault('timeouts', {})
        self.consumer.settings['timeouts']['httpbin'] = 1234.5
        yield self.process_message()
        self.consumer.http.fetch.assert_called_once_with(
            mock.ANY, headers=mock.ANY, method=mock.ANY, body=mock.ANY,
            raise_error=False, request_timeout=1234.5)

    @testing.gen_test
    def test_that_function_timeout_is_preferred(self):
        self.consumer.settings.setdefault('timeouts', {})
        self.consumer.settings['timeouts']['fetch-stats'] = 1234.5
        self.consumer.settings['timeouts']['httpbin'] = 9876.5
        yield self.process_message()
        self.consumer.http.fetch.assert_called_once_with(
            mock.ANY, headers=mock.ANY, method=mock.ANY, body=mock.ANY,
            raise_error=False, request_timeout=1234.5)



import unittest

import helper.config
import mock

from vetoes import config


class FeatureFlagMixinTests(unittest.TestCase):

    def test_that_flags_are_processed_during_initialize(self):
        settings = helper.config.Data({
            'features': {'on': 'on', 'off': 'false'}
        })
        consumer = config.FeatureFlagMixin(settings, mock.Mock())
        self.assertTrue(consumer.feature_flags['on'])
        self.assertFalse(consumer.feature_flags['off'])

    def test_that_invalid_flags_arg_ignored(self):
        settings = helper.config.Data({
            'features': {'one': 'not valid', 'two': None}
        })
        consumer = config.FeatureFlagMixin(settings, mock.Mock())
        self.assertEqual(consumer.feature_flags, {})



import json
import select
import socket

from rejected import consumer
from tornado import gen, httpclient, httputil


class HTTPServiceMixin(consumer.Consumer):
    """
    Mix this in to add the :meth:`.call_http_service` method.

    :keyword dict service_map: mapping from logical "function"
        to HTTP service.
    :param args: these are passed to :meth:`rejected.consumer.Consumer`
        as-is
    :param kwargs: these are passed to :meth:`rejected.consumer.Consumer`
        as-is

    The primary use of this mix-in is to make sending HTTP requests
    easier and safer.  The :meth:`.call_http_service` method uses the
    :meth:`.get_service_url` to build the request URL for a named
    **semantic function**, send the request to a specific **HTTP service**,
    and perform some "opinionated" processing on the response.  The
    separation of *semantic function* and *HTTP service* may seem a bit
    confusing at first.  They are used to provide concise logging and
    well-named metrics.  The semantic function describes the *action
    being performed* and the service is the *actor performing the action*.

    The mapping from *semantic function* to HTTP service is handled by
    the `service_map` passed into the initializer.  The mapping value is
    the **HTTP service** which is passed into :meth:`.get_service_url` to
    construct the request URL.

    HTTP client behavior is controlled via consumer level configuration under
    the ``vetoes`` key. The following options are available:

    +-----------------+-----------------------------------------------------+
    | Key             | Description                                         |
    +=================+=====================================================+
    | max_clients     | The max # of simultaneous requests that can be made |
    +-----------------+-----------------------------------------------------+
    | connect_timeout | Timeout for initial connection in seconds           |
    +-----------------+-----------------------------------------------------+
    | request_time    | Timeout for entire request in seconds               |
    +-----------------+-----------------------------------------------------+

    *Example Configuration:*

    .. code:: yaml

        Application:
          Consumers:
            example:
              consumer: rejected.example.Consumer
              connections:
                - name: rabbitmq1
              qty: 2
              queue: generated_messages
              config:
                vetoes:
                  max_clients: 10
                  connect_timeout: 5.0
                  request_timeout: 30.0

    .. attribute:: http_headers

       :class:`tornado.httputil.HTTPHeaders` instance of headers
       that are included in every request.  This set is empty.

    .. attribute:: http

       :class:`tornado.httpclient.AsyncHTTPClient` used to make
       requests.  The initializer sets the ``connect_timeout``
       and ``request_timeout`` in ``self.http.defaults``.

    """

    def __init__(self, *args, **kwargs):
        self.__service_map = kwargs.pop('service_map')
        super(HTTPServiceMixin, self).__init__(*args, **kwargs)
        self.http_headers = httputil.HTTPHeaders()
        settings = self._settings.get('vetoes', {})
        self.http = httpclient.AsyncHTTPClient(
            force_instance=True,
            max_clients=settings.get('max_clients'))
        self.http.defaults['connect_timeout'] = \
            settings.get('connect_timeout', 5.0)
        self.http.defaults['request_timeout'] = \
            settings.get('request_timeout', 30.0)

    @gen.coroutine
    def call_http_service(self, function, method, *path, **kwargs):
        """
        Send a HTTP request to a service.

        :param str function: the function to invoke.  The service
            is determined based on the ``service_map`` established
            during initialization.
        :param str method: HTTP method to invoke.
        :param path: path elements to the HTTP resource.
        :keyword dict headers: optional set of headers to include
            in the message.  These are in addition to :attr:`http_headers`.
        :keyword json: optional body to send in the message.
            If this keyword is included, then the value is JSON encoded
            before being used as the body.
        :type json: dict or list
        :keyword bool raise_error: if this keyword is included and
            set to :data:`False`, then HTTP errors will be returned
            instead of raised as exceptions.
        :keyword str url: if this keyword is included then it is used
            as-is instead of doing a service lookup.
        :param kwargs: additional keyword arguments are passed to
            :meth:`tornado.httpclient.AsyncHTTPClient.fetch`.

        :returns: a :class:`tornado.httpclient.HTTPResponse` instance
        :rtype: tornado.httpclient.HTTPResponse

        :raises: tornado.httpclient.HTTPError if a HTTP error
            occurs
        :raises: rejected.consumer.ProcessingException if a low-level
            socket error occurs or a retry-able HTTP result is returned

        """
        headers = httputil.HTTPHeaders()
        if self.correlation_id:
            headers['Correlation-Id'] = self.correlation_id
        headers.update(self.http_headers)
        headers.update(kwargs.pop('headers', {}))

        if 'json' in kwargs:
            if kwargs['json'] is not None:
                headers.setdefault('Content-Type', 'application/json')
                kwargs['body'] = json.dumps(kwargs['json']).encode('utf-8')
            kwargs.pop('json', None)

        if headers:
            kwargs['headers'] = headers

        if 'url' in kwargs:
            url = kwargs.pop('url')
            service = function
        else:
            service = self.__service_map[function]
            url = self.get_service_url(
                service, *path, query_args=kwargs.pop('query_args', None))

        if 'request_timeout' not in kwargs and hasattr(self, 'get_timeout'):
            kwargs['request_timeout'] = self.get_timeout(
                function, self.get_timeout(service))

        self.set_sentry_context('service_invoked', service)

        self.logger.debug('sending %s request to %s', method, url)
        raise_error = kwargs.pop('raise_error', True)
        iol = self._channel.connection.ioloop
        start_time = iol.time()

        try:
            response = yield self.http.fetch(url, method=method,
                                             raise_error=False, **kwargs)
            self.statsd_add_timing(
                'http.{0}.{1}'.format(function, response.code),
                response.request_time)

        except (OSError, select.error, socket.error) as e:
            self.logger.exception('%s to %s failed', method, url)
            self.statsd_add_timing(
                'http.{0}.timeout'.format(function),
                iol.time() - start_time)
            self.statsd_incr(
                'errors.socket.{0}'.format(getattr(e, 'errno', 'unknown')))
            raise consumer.ProcessingException(
                '{0} connection failure - {1}'.format(service, e))

        finally:
            self.unset_sentry_context('service_invoked')

        if response.code == 429:
            raise consumer.ProcessingException(
                '{0} is rate limiting requests'.format(service))

        if response.code == 599:
            raise consumer.ProcessingException(
                '{0} timed out for {1} to {2}'.format(service, method, url))

        if raise_error:
            response.rethrow()

        raise gen.Return(response)

    def get_service_url(self, service, *path, **kwargs):
        """
        Build a request URL for a specific service.

        :param str service: name of the service to invoke
        :param path: resource path elements
        :keyword query_args: optional query parameters to include in the URL
        :type query_args: dict or None

        :returns: the request URL
        :rtype: str

        .. note::

           You are required to override this method in your consumer.
           The base implementation simply raises an exception.

        """
        raise NotImplementedError



import unittest

import helper.config
import mock

from vetoes import config


class TimeoutConfigurationTests(unittest.TestCase):

    def test_that_default_timeout_is_500ms(self):
        settings = helper.config.Data({})
        consumer = config.TimeoutConfigurationMixin(settings, mock.Mock())
        self.assertAlmostEqual(consumer.get_timeout('whatever'), 0.5)

    def test_that_default_timeout_is_configurable(self):
        settings = helper.config.Data({'timeouts': {'default': '1.0'}})
        consumer = config.TimeoutConfigurationMixin(settings, mock.Mock())
        self.assertAlmostEqual(consumer.get_timeout('whatever'), 1.0)

    def test_that_named_timeout_works(self):
        settings = helper.config.Data({'timeouts': {'whatever': '1.0'}})
        consumer = config.TimeoutConfigurationMixin(settings, mock.Mock())
        self.assertAlmostEqual(consumer.get_timeout('whatever'), 1.0)



#!/usr/bin/env python
#

import setuptools

import vetoes


version = vetoes.version
try:
    with open('LOCAL-VERSION') as version_file:
        version += version_file.readline().strip()
except IOError:
    pass


setuptools.setup(
    name='vetoes',
    version=version,
    description='Rejected companion library',
    packages=['vetoes'],
    url='https://github.com/dave-shawley/vetoes',
    author='Dave Shawley',
    author_email='daveshawley+python@gmail.com',
    long_description='\n'+open('README.rst').read(),
    install_requires=['rejected'],
    platforms='any',
    classifiers=['Intended Audience :: Developers',
                'License :: OSI Approved :: BSD License',
                'Operating System :: OS Independent',
                'Programming Language :: Python',
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 3',
                'Development Status :: 4 - Beta'],
)



import distutils.util

from rejected import consumer


class FeatureFlagMixin(consumer.Consumer):
    """
    Mix this in to parse ``self.settings['features']``.

    Each flag in the ``features`` consumer setting is parsed using
    :func:`distutils.util.strtobool` and if it is parseable, then
    ``self.feature_flags[name]`` is set to the parsed result.

    For example, if your rejected configuration looks like:

    .. code-block:: yaml

       Application:
         Consumers:
           my-consumer:
             config:
               features:
                 frobinicate: on
                 defenestrate: no

    Then your consumer can expect ``feature_flags`` to be:

    .. code-block:: python

       feature_flags = {'frobinicate': True,
                        'defenestrate': False}

    """

    def __init__(self, *args, **kwargs):
        self.feature_flags = {}
        super(FeatureFlagMixin, self).__init__(*args, **kwargs)

    def initialize(self):
        super(FeatureFlagMixin, self).initialize()
        self._read_feature_flags()

    def _read_feature_flags(self):
        """Process ``self.settings['features']`` as a set of named flags."""
        flags = self.settings.get('features', {})
        for k, v in flags.items():
            try:
                parsed = bool(distutils.util.strtobool(v))
                self.feature_flags[k] = parsed
                self.logger.debug('feature %s is %s', k,
                                  'enabled' if parsed else 'disabled')

            except (AttributeError, ValueError) as e:
                self.logger.warning('failed to parse feature flag %s=%s - %r',
                                    k, v, e)


class TimeoutConfigurationMixin(consumer.Consumer):
    """
    Mix this in to parse ``self.settings['timeouts']``.

    The members from the timeouts dictionary is exposed by the
    :meth:`get_timeout` method.

    For example, if your rejected configuration looks like:

    .. code-block:: yaml

       Applications:
         Consumers:
           my-consumer:
             config:
               timeouts:
                 default: 0.1
                 dynamodb: 1.5

    Then your consumer can expect the following to hold true:

    .. code-block:: python

       self.get_timeout('dynamodb') == 1.5
       self.get_timeout('something-else') == 0.1

    If you omit the "default" key, then it will fallback to 0.5s.

    """

    def initialize(self):
        super(TimeoutConfigurationMixin, self).initialize()
        timeouts = self.settings.get('timeouts', {})
        self.__default_timeout = float(timeouts.get('default', '0.5'))

    def get_timeout(self, name, default=None):
        """
        Get the timeout named `name` from the consumer configuration.

        :param str name: the name of the timeout
        :param default: optional default value.  If this is omitted
            then the ``get_timeout('default', 0.5)`` will be returned
        :return: the appropriate timeout value in seconds
        :rtype: float

        """
        timeouts = self.settings.get('timeouts', {})
        return float(timeouts.get(name, default or self.__default_timeout))
